package org.itqop.chatit.utils;

import net.minecraftforge.common.ForgeConfigSpec;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.event.config.ModConfigEvent;
import org.itqop.chatit.ChatIT;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Mod.EventBusSubscriber(modid = ChatIT.MODID, bus = Mod.EventBusSubscriber.Bus.MOD)
public class Config {

    private static final ForgeConfigSpec.Builder BUILDER = new ForgeConfigSpec.Builder();
    private static final Logger LOGGER = LoggerFactory.getLogger(Config.class);

    public static ForgeConfigSpec COMMON_CONFIG;

    public static ForgeConfigSpec.ConfigValue<String> HOST_API;
    public static ForgeConfigSpec.BooleanValue DEFAULT_ADULT;
    public static ForgeConfigSpec.BooleanValue USE_REGEX;
    public static ForgeConfigSpec.DoubleValue PROFANITY_THRESHOLD;
    public static ForgeConfigSpec.ConfigValue<String> PROFANITY_REGEX;
    public static ForgeConfigSpec.ConfigValue<String> MESSAGE_ADULT;
    public static ForgeConfigSpec.ConfigValue<String> MESSAGE_LOCAL;

    public static ForgeConfigSpec.ConfigValue<String> PROFANITY_MODE;

    static {
        BUILDER.comment("Настройки ChatIT");

        HOST_API = BUILDER
                .comment("URL API для проверки мата")
                .define("host_api", "");

        DEFAULT_ADULT = BUILDER
                .comment("Значение adult по умолчанию для новых игроков")
                .define("default_adult", false);

        USE_REGEX = BUILDER
                .comment("Использовать регулярное выражение при недоступности API")
                .define("regex", false);

        PROFANITY_THRESHOLD = BUILDER
                .comment("Порог для определения мата (0.0 - 1.0)")
                .defineInRange("profanity_threshold", 0.75, 0.0, 1.0); // Дефолтное значение 0.5

        PROFANITY_REGEX = BUILDER
                .comment("Регулярное выражение для проверки мата")
                .define("profanity_regex", "(?iu)\\b(?:(?:(?:у|[нз]а|(?:хитро|не)?вз?[ыьъ]|с[ьъ]|(?:и|ра)[зс]ъ?|(?:.\\B)+?[оаеи-])-?)?(?:[её](?:б(?!о[рй]|рач)|п[уа](?:ц|тс))|и[пб][ае][тцд][ьъ]).*?|(?:(?:н[иеа]|(?:ра|и)[зс]|[зд]?[ао](?:т|дн[оа])?|с(?:м[еи])?|а[пб]ч|в[ъы]?|пр[еи])-?)?ху(?:[яйиеёю]|л+и(?!ган)).*?|бл(?:[эя]|еа?)(?:[дт][ьъ]?)?|\\S*?(?:п(?:[иеё]зд|ид[аое]?р|ед(?:р(?!о)|[аое]р|ик)|охую)|бля(?:[дбц]|тс)|[ое]ху[яйиеё]|хуйн).*?|(?:о[тб]?|про|на|вы)?м(?:анд(?:[ауеыи](?:л(?:и[сзщ])?[ауеиы])?|ой|[ао]в.*?|юк(?:ов|[ауи])?|е[нт]ь|ища)|уд(?:[яаиое].+?|е?н(?:[ьюия]|ей))|[ао]л[ао]ф[ьъ](?:[яиюе]|[еёо]й))|елд[ауые].*?|ля[тд]ь|(?:[нз]а|по)х)\\b"); // Дефолтное регулярное выражение

        MESSAGE_ADULT = BUILDER
                .comment("Сообщение запрета на маты.")
                .define("message_adult", "Вы пытаетесь использовать ненормативную лексику в своем сообщении.");

        MESSAGE_LOCAL = BUILDER
                .comment("Сообщение предупреждения о том, что никого рядом нет")
                .define("message_local", "Вас никто не услышал, поставьте ! перед сообщением.");

        // Определение нового параметра profanity_mode
        PROFANITY_MODE = BUILDER
                .comment("Режим проверки мата: off, regex, api")
                .define("profanity_mode", "regex"); // Дефолтное значение: api

        COMMON_CONFIG = BUILDER.build();
    }

    // Статические переменные для доступа к настройкам
    public static String hostApi;
    public static boolean defaultAdult;
    public static boolean useRegex;
    public static double profanityThreshold;
    public static String profanityRegex;
    public static String messageAdult;
    public static String messageLocal;
    public static ProfanityMode profanityMode;

    @SubscribeEvent
    public static void onLoad(final ModConfigEvent event) {
        hostApi = HOST_API.get();
        defaultAdult = DEFAULT_ADULT.get();
        useRegex = USE_REGEX.get();
        profanityThreshold = PROFANITY_THRESHOLD.get();
        profanityRegex = PROFANITY_REGEX.get();
        messageAdult = MESSAGE_ADULT.get();
        messageLocal = MESSAGE_LOCAL.get();

        // Чтение и установка режима проверки мата
        String modeString = PROFANITY_MODE.get().toLowerCase();
        try {
            profanityMode = ProfanityMode.fromString(modeString);
        } catch (IllegalArgumentException e) {
            profanityMode = ProfanityMode.REGEX; // Значение по умолчанию
            LOGGER.warn("Неверный режим проверки мата в конфигурации: {}. Используется режим API.", modeString);
        }
    }

    /**
     * Метод для установки нового режима проверки мата.
     *
     * @param mode Новый режим.
     */
    public static void setProfanityMode(ProfanityMode mode) {
        profanityMode = mode;
        PROFANITY_MODE.set(mode.name().toLowerCase());

        COMMON_CONFIG.save();
        LOGGER.info("Режим проверки мата установлен на: {} и конфигурация сохранена.", mode.name().toLowerCase());
    }


    /**
     * Метод для получения текущего режима проверки мата.
     *
     * @return Текущий режим.
     */
    public static ProfanityMode getProfanityMode() {
        return profanityMode;
    }
}
