package org.itqop.chatit.utils;

import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.server.network.ServerGamePacketListenerImpl;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.net.URL;


public class Utils {

    private static final DateTimeFormatter LOG_TIME_FORMAT = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    private static final String PRIVATE_MESSAGES_LOG = "logs/private_messages.log";
    private static final String GLOBAL_MESSAGES_LOG = "logs/global_messages.log";

    /**
     * Метод для получения IP-адреса игрока.
     *
     * @param player Игрок, чей IP требуется получить.
     * @return IP-адрес игрока в строковом формате.
     */
    private static String getPlayerIp(ServerPlayer player) {
        ServerGamePacketListenerImpl connection = player.connection;

        InetSocketAddress address = (InetSocketAddress) connection.connection.getRemoteAddress();

        InetAddress inetAddress = address.getAddress();
        if (inetAddress == null) {
            return "Unknown IP";
        }

        if (inetAddress instanceof Inet4Address) {
            return inetAddress.getHostAddress();
        } else if (inetAddress instanceof Inet6Address) {
            String ip = inetAddress.getHostAddress();
            int percentIndex = ip.indexOf('%');
            if (percentIndex != -1) {
                ip = ip.substring(0, percentIndex);
            }
            return ip;
        } else {
            return inetAddress.getHostAddress();
        }
    }

    /**
     * Метод для логирования приватных сообщений в файл.
     *
     * @param sender  Игрок, отправляющий сообщение.
     * @param target  Целевой игрок.
     * @param message Текст сообщения.
     */
    public static void logPrivateMessage(ServerPlayer sender, ServerPlayer target, Component message) {
        String time = LocalDateTime.now().format(LOG_TIME_FORMAT);
        String senderIp = getPlayerIp(sender);
        String senderName = sender.getName().getString();
        String targetName = target.getName().getString();
        String messageText = message.getString();
        String logEntry = String.format("[%s] [%s] - %s -> %s: %s%n", time, senderIp, senderName, targetName, messageText);

        try (BufferedWriter writer = new BufferedWriter(new FileWriter(PRIVATE_MESSAGES_LOG, true))) {
            writer.write(logEntry);
        } catch (IOException e) {
            //LOGGER.error("Не удалось записать приватное сообщение в лог.", e);
        }
    }

    /**
     * Метод для логирования глобальных сообщений в файл.
     *
     * @param message Глобальное сообщение в формате MutableComponent.
     */
    public static void logGlobalMessage(MutableComponent message) {
        String time = LocalDateTime.now().format(LOG_TIME_FORMAT);
        String messageText = message.getString();
        String logEntry = String.format("[%s] %s%n", time, messageText);

        try (BufferedWriter writer = new BufferedWriter(new FileWriter(GLOBAL_MESSAGES_LOG, true))) {
            writer.write(logEntry);
        } catch (IOException e) {
            //LOGGER.error("Не удалось записать глобальное сообщение в лог.", e);
        }
    }

    /**
     * Метод для проверки валидности URL.
     * В будущем сюда можно добавить дополнительные проверки, например, DGA.
     *
     * @param url Строка с URL для проверки.
     * @return true, если URL валиден, иначе false.
     */
    public static boolean isValidUrl(String url) {
        try {
            new URL(url).toURI(); // Проверяем синтаксис URL
            return true;
        } catch (Exception e) {
            return false;
        }
    }

}
